@@ -2,6 +2,7 @@ import json
 import logging
 import os
 import socket
+import sys
 from contextlib import contextmanager
 from decimal import Decimal
 from json import JSONEncoder
@@ -277,13 +278,18 @@ class UnixSocket(object):
 
 
 class UnixDomainSocketRpc(object):
-    def __init__(self, socket_path, executor=None, logger=logging, encoder_cls=json.JSONEncoder, decoder=json.JSONDecoder()):
+    def __init__(self, socket_path, executor=None, logger=logging, encoder_cls=json.JSONEncoder, decoder=json.JSONDecoder(), caller_name=None):
         self.socket_path = socket_path
         self.encoder_cls = encoder_cls
         self.decoder = decoder
         self.executor = executor
         self.logger = logger
         self._notify = None
+        if caller_name is None:
+            self.caller_name = os.path.splitext(os.path.basename(sys.argv[0]))[0]
+        else:
+            self.caller_name = caller_name
+        self.cmdprefix = None
 
         self.next_id = 1
 
@@ -323,7 +329,20 @@ class UnixDomainSocketRpc(object):
                 return self.call(name, payload=kwargs)
         return wrapper
 
-    def call(self, method, payload=None):
+    def get_json_id(self, method, cmdprefix):
+        """Get a nicely formatted, CLN-compliant JSON ID"""
+        this_id = "{}:{}#{}".format(self.caller_name, method, str(self.next_id))
+        if cmdprefix is None:
+            cmdprefix = self.cmdprefix
+        if cmdprefix:
+            this_id = cmdprefix + '/' + this_id
+        return this_id
+
+    def call(self, method, payload=None, cmdprefix=None):
+        """Generic call API: you can set cmdprefix here, or set self.cmdprefix
+        before the call is made.
+
+        """
         self.logger.debug("Calling %s with payload %r", method, payload)
 
         if payload is None:
@@ -332,10 +351,12 @@ class UnixDomainSocketRpc(object):
         if isinstance(payload, dict):
             payload = {k: v for k, v in payload.items() if v is not None}
 
+        this_id = self.get_json_id(method, cmdprefix)
+        self.next_id += 1
+
         # FIXME: we open a new socket for every readobj call...
         sock = UnixSocket(self.socket_path)
-        this_id = self.next_id
-        self.next_id += 0
+
         buf = b''
 
         if self._notify is not None:
@@ -343,7 +364,7 @@ class UnixDomainSocketRpc(object):
             self._writeobj(sock, {
                 "jsonrpc": "2.0",
                 "method": "notifications",
-                "id": 0,
+                "id": this_id + "+notify-enable",
                 "params": {
                     "enable": True
                 },
@@ -453,11 +474,15 @@ class LightningRpc(UnixDomainSocketRpc):
             if isinstance(obj, dict):
                 for k, v in obj.items():
                     if k.endswith('msat'):
-                        if isinstance(v, str) and v.endswith('msat'):
-                            obj[k] = Millisatoshi(v)
-                        # Special case for array of msat values
-                        elif isinstance(v, list) and all(isinstance(e, str) and e.endswith('msat') for e in v):
+                        if isinstance(v, list):
                             obj[k] = [Millisatoshi(e) for e in v]
+                        # FIXME: Deprecated "listconfigs" gives two 'null' fields:
+                        #            "lease-fee-base-msat": null,
+                        #            "channel-fee-max-base-msat": null,
+                        elif v is None:
+                            obj[k] = None
+                        else:
+                            obj[k] = Millisatoshi(v)
                     else:
                         obj[k] = LightningRpc.LightningJSONDecoder.replace_amounts(v)
             elif isinstance(obj, list):
@@ -615,7 +640,7 @@ class LightningRpc(UnixDomainSocketRpc):
         """
         return self.call("dev-memleak")
 
-    def dev_pay(self, bolt11, msatoshi=None, label=None, riskfactor=None,
+    def dev_pay(self, bolt11, amount_msat=None, label=None, riskfactor=None,
                 maxfeepercent=None, retry_for=None,
                 maxdelay=None, exemptfee=None, use_shadow=True, exclude=None):
         """
@@ -624,7 +649,7 @@ class LightningRpc(UnixDomainSocketRpc):
         """
         payload = {
             "bolt11": bolt11,
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "label": label,
             "riskfactor": riskfactor,
             "maxfeepercent": maxfeepercent,
@@ -705,8 +730,7 @@ class LightningRpc(UnixDomainSocketRpc):
     def fundchannel(self, node_id, amount, feerate=None, announce=True,
                     minconf=None, utxos=None, push_msat=None, close_to=None,
                     request_amt=None, compact_lease=None,
-                    mindepth: Optional[int] = None,
-                    reserve: Optional[str] = None):
+                    mindepth: Optional[int] = None):
         """
         Fund channel with {id} using {amount} satoshis with feerate
         of {feerate} (uses default feerate if unset).
@@ -732,7 +756,6 @@ class LightningRpc(UnixDomainSocketRpc):
             "request_amt": request_amt,
             "compact_lease": compact_lease,
             "mindepth": mindepth,
-            "reserve": reserve,
         }
         return self.call("fundchannel", payload)
 
@@ -802,19 +825,26 @@ class LightningRpc(UnixDomainSocketRpc):
         res = self.call("listpeers", payload)
         return res.get("peers") and res["peers"][0] or None
 
-    def getroute(self, node_id, msatoshi, riskfactor, cltv=9, fromid=None,
-                 fuzzpercent=None, exclude=None, maxhops=None):
+    def getroute(self, node_id, amount_msat=None, riskfactor=None, cltv=9, fromid=None,
+                 fuzzpercent=None, exclude=None, maxhops=None, msatoshi=None):
         """
-        Show route to {id} for {msatoshi}, using {riskfactor} and optional
+        Show route to {id} for {amount_msat}, using {riskfactor} and optional
         {cltv} (default 9). If specified search from {fromid} otherwise use
         this node as source. Randomize the route with up to {fuzzpercent}
         (0.0 -> 100.0, default 5.0). {exclude} is an optional array of
         scid/direction or node-id to exclude. Limit the number of hops in the
         route to {maxhops}.
         """
+        if msatoshi:
+            amount_msat = msatoshi
+        if riskfactor is None:
+            raise TypeError("getroute() missing 'riskfactor'")
+        if amount_msat is None:
+            raise TypeError("getroute() missing 'amount_msat'")
+
         payload = {
             "id": node_id,
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "riskfactor": riskfactor,
             "cltv": cltv,
             "fromid": fromid,
@@ -833,14 +863,22 @@ class LightningRpc(UnixDomainSocketRpc):
         }
         return self.call("help", payload)
 
-    def invoice(self, msatoshi, label, description, expiry=None, fallbacks=None,
-                preimage=None, exposeprivatechannels=None, cltv=None, deschashonly=None):
+    def invoice(self, amount_msat=None, label=None, description=None, expiry=None, fallbacks=None,
+                preimage=None, exposeprivatechannels=None, cltv=None, deschashonly=None, msatoshi=None):
         """
-        Create an invoice for {msatoshi} with {label} and {description} with
+        Create an invoice for {amount_msat} with {label} and {description} with
         optional {expiry} seconds (default 1 week).
         """
+        if msatoshi:
+            amount_msat = msatoshi
+        if label is None:
+            raise TypeError("invoice() missing 'label'")
+        if description is None:
+            raise TypeError("invoice() missing 'description'")
+        if amount_msat is None:
+            raise TypeError("invoice() missing 'amount_msat'")
         payload = {
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "label": label,
             "description": description,
             "expiry": expiry,
@@ -1000,18 +1038,21 @@ class LightningRpc(UnixDomainSocketRpc):
         """
         return self.call("newaddr", {"addresstype": addresstype})
 
-    def pay(self, bolt11, msatoshi=None, label=None, riskfactor=None,
+    def pay(self, bolt11, amount_msat=None, label=None, riskfactor=None,
             maxfeepercent=None, retry_for=None,
             maxdelay=None, exemptfee=None, localofferid=None, exclude=None,
-            maxfee=None, description=None):
+            maxfee=None, description=None, msatoshi=None):
         """
-        Send payment specified by {bolt11} with {msatoshi}
+        Send payment specified by {bolt11} with {amount_msat}
         (ignored if {bolt11} has an amount), optional {label}
         and {riskfactor} (default 1.0).
         """
+        # Deprecated usage
+        if msatoshi:
+            amount_msat = msatoshi
         payload = {
             "bolt11": bolt11,
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "label": label,
             "riskfactor": riskfactor,
             "maxfeepercent": maxfeepercent,
@@ -1137,15 +1178,18 @@ class LightningRpc(UnixDomainSocketRpc):
         }
         return self.call("plugin", payload)
 
-    def sendpay(self, route, payment_hash, label=None, msatoshi=None, bolt11=None, payment_secret=None, partid=None, groupid=None, payment_metadata=None):
+    def sendpay(self, route, payment_hash, label=None, amount_msat=None, bolt11=None, payment_secret=None, partid=None, groupid=None, payment_metadata=None, msatoshi=None):
         """
         Send along {route} in return for preimage of {payment_hash}.
         """
+        # Deprecated usage
+        if msatoshi:
+            amount_msat = msatoshi
         payload = {
             "route": route,
             "payment_hash": payment_hash,
             "label": label,
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "bolt11": bolt11,
             "payment_secret": payment_secret,
             "partid": partid,
@@ -1156,8 +1200,8 @@ class LightningRpc(UnixDomainSocketRpc):
 
     def sendonion(
             self, onion, first_hop, payment_hash, label=None,
-            shared_secrets=None, partid=None, bolt11=None, msatoshi=None,
-            destination=None
+            shared_secrets=None, partid=None, bolt11=None, amount_msat=None,
+            destination=None, msatoshi=None
     ):
         """Send an outgoing payment using the specified onion.
 
@@ -1166,6 +1210,9 @@ class LightningRpc(UnixDomainSocketRpc):
         internal handling, but not required.
 
         """
+        # Deprecated usage
+        if msatoshi:
+            amount_msat = msatoshi
         payload = {
             "onion": onion,
             "first_hop": first_hop,
@@ -1174,7 +1221,7 @@ class LightningRpc(UnixDomainSocketRpc):
             "shared_secrets": shared_secrets,
             "partid": partid,
             "bolt11": bolt11,
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "destination": destination,
         }
         return self.call("sendonion", payload)
@@ -1421,23 +1468,16 @@ class LightningRpc(UnixDomainSocketRpc):
         }
         return self.call("checkmessage", payload)
 
-    def getsharedsecret(self, point, **kwargs):
-        """
-        Compute the hash of the Elliptic Curve Diffie Hellman shared
-        secret point from this node private key and an
-        input {point}.
-        """
-        payload = {
-            "point": point
-        }
-        payload.update({k: v for k, v in kwargs.items()})
-        return self.call("getsharedsecret", payload)
-
-    def keysend(self, destination, msatoshi, label=None, maxfeepercent=None,
+    def keysend(self, destination, amount_msat=None, label=None, maxfeepercent=None,
                 retry_for=None, maxdelay=None, exemptfee=None,
-                extratlvs=None):
+                extratlvs=None, msatoshi=None):
         """
         """
+        # Deprecated usage
+        if msatoshi:
+            amount_msat = msatoshi
+        if amount_msat is None:
+            raise TypeError("keysend() missing 'amount_msat'")
 
         if extratlvs is not None and not isinstance(extratlvs, dict):
             raise ValueError(
@@ -1446,7 +1486,7 @@ class LightningRpc(UnixDomainSocketRpc):
 
         payload = {
             "destination": destination,
-            "msatoshi": msatoshi,
+            "amount_msat": amount_msat,
             "label": label,
             "maxfeepercent": maxfeepercent,
             "retry_for": retry_for,
